// This file is part of cxsd, copyright (c) 2016 BusFaster Ltd.
// Released under the MIT license, see LICENSE.
"use strict";
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var Exporter_1 = require('./Exporter');
var Namespace_1 = require('../Namespace');
var MemberRef_1 = require('../MemberRef');
var Type_1 = require('../Type');
var JS = (function (_super) {
    __extends(JS, _super);
    function JS() {
        _super.apply(this, arguments);
        this.construct = JS;
    }
    JS.prototype.writeImport = function (shortName, relativePath, absolutePath) {
        return ('var ' +
            shortName +
            ' = require(' +
            "'" + relativePath + "'" +
            ');');
    };
    JS.prototype.writeMember = function (ref, typeNumTbl, importNumTbl) {
        var member = ref.member;
        var name = ref.safeName;
        if (member.name != name)
            name += ':' + member.name;
        var flags = 0;
        if (ref.min < 1)
            flags |= MemberRef_1.MemberRef.optionalFlag;
        if (ref.max > 1)
            flags |= MemberRef_1.MemberRef.arrayFlag;
        if (member.name == '*')
            flags |= MemberRef_1.MemberRef.anyFlag;
        var memberTypeList = member.typeList.map(function (memberType) {
            return typeNumTbl[memberType.surrogateKey];
        });
        return ('[' +
            "'" + name + "', " +
            flags + ', ' +
            '[' + memberTypeList.join(', ') + ']' +
            ((member.namespace != this.namespace) ? ', ' + importNumTbl[member.namespace.id] : '') +
            ']');
    };
    JS.prototype.writeType = function (type, typeNumTbl, importNumTbl) {
        var childSpecList = [];
        var attributeSpecList = [];
        var parentNum = 0;
        var flags = 0;
        if (type.primitiveType)
            flags |= Type_1.Type.primitiveFlag;
        if (type.isPlainPrimitive)
            flags |= Type_1.Type.plainPrimitiveFlag;
        if (type.isList) {
            flags |= Type_1.Type.listFlag | Type_1.Type.primitiveFlag | Type_1.Type.plainPrimitiveFlag;
            parentNum = typeNumTbl[type.childList[0].member.typeList[0].surrogateKey];
        }
        else {
            if (type.childList) {
                for (var _i = 0, _a = type.childList; _i < _a.length; _i++) {
                    var member = _a[_i];
                    childSpecList.push(this.writeMember(member, typeNumTbl, importNumTbl));
                }
            }
            if (type.attributeList) {
                for (var _b = 0, _c = type.attributeList; _b < _c.length; _b++) {
                    var member = _c[_b];
                    attributeSpecList.push(this.writeMember(member, typeNumTbl, importNumTbl));
                }
            }
            if (type.parent)
                parentNum = typeNumTbl[type.parent.surrogateKey];
        }
        return ('\n\t[' +
            flags + ', ' +
            parentNum + ', ' +
            '[' + childSpecList.join(', ') + '], ' +
            '[' + attributeSpecList.join(', ') + ']' +
            ']');
    };
    /** Output namespace contents to the given cache key. */
    JS.prototype.writeContents = function () {
        var doc = this.doc;
        var namespace = doc.namespace;
        var typeNumTbl = {};
        var memberNumTbl = {};
        var typeNum = 1;
        var memberNum = 1;
        var importTbl = namespace.getUsedImportTbl();
        var importSpecList = [];
        var importNumTbl = {};
        var num = 0;
        for (var _i = 0, _a = Object.keys(importTbl); _i < _a.length; _i++) {
            var importName = _a[_i];
            var otherNamespaceId = importTbl[importName].id;
            var content = namespace.importContentTbl[otherNamespaceId];
            var importTypeNameList = [];
            var importMemberNameList = [];
            for (var _b = 0, _c = Object.keys(content.typeTbl || {}).sort(); _b < _c.length; _b++) {
                var name = _c[_b];
                var type = content.typeTbl[name];
                importTypeNameList.push("'" + type.safeName + "'");
                typeNumTbl[type.surrogateKey] = typeNum++;
            }
            for (var _d = 0, _e = Object.keys(content.memberTbl || {}).sort(); _d < _e.length; _d++) {
                var name = _e[_d];
                var member = content.memberTbl[name];
                importMemberNameList.push("'" + member.name + "'");
                memberNumTbl[member.surrogateKey] = memberNum++;
            }
            importSpecList.push('\n\t' + '[' + importName + ', ' +
                '[' + importTypeNameList.join(', ') + '], ' +
                '[' + importMemberNameList.join(', ') + ']' +
                ']');
            importNumTbl[otherNamespaceId] = num++;
        }
        var exportedTypeList = [];
        var hiddenTypeList = [];
        var typeSpecList = [];
        for (var _f = 0, _g = namespace.typeList; _f < _g.length; _f++) {
            var type = _g[_f];
            if (!type)
                continue;
            var isExported = (namespace.typeStateList[type.surrogateKey] == Namespace_1.TypeState.exported);
            if (isExported)
                exportedTypeList.push(type);
            else
                hiddenTypeList.push(type);
        }
        exportedTypeList.sort(function (a, b) { return a.safeName.localeCompare(b.safeName); });
        hiddenTypeList.sort(function (a, b) { return a.safeName.localeCompare(b.safeName); });
        var typeList = exportedTypeList.concat(hiddenTypeList);
        for (var _h = 0, typeList_1 = typeList; _h < typeList_1.length; _h++) {
            var type = typeList_1[_h];
            typeNumTbl[type.surrogateKey] = typeNum++;
        }
        var parentNum;
        typeSpecList.push(this.writeType(namespace.doc, typeNumTbl, importNumTbl));
        for (var _j = 0, typeList_2 = typeList; _j < typeList_2.length; _j++) {
            var type = typeList_2[_j];
            typeSpecList.push(this.writeType(type, typeNumTbl, importNumTbl));
        }
        var exportSpecList = [];
        for (var _k = 0, exportedTypeList_1 = exportedTypeList; _k < exportedTypeList_1.length; _k++) {
            var type = exportedTypeList_1[_k];
            name = type.safeName;
            if (type.name && type.name != name)
                name += ':' + type.name;
            exportSpecList.push('\n\t' + "'" + name + "'");
        }
        return ([].concat([
            'var cxml = require("cxml");',
        ], this.writeHeader(), [
            '',
            'cxml.register(' +
                "'" + namespace.name + "', " +
                'exports, ' +
                '[' + importSpecList.join(',') + '\n], ' +
                '[' + exportSpecList.join(',') + '\n], ' +
                '[' + typeSpecList.join(',') + '\n]' +
                ');'
        ]).join('\n'));
    };
    JS.prototype.getOutName = function (name) {
        return (name + '.js');
    };
    return JS;
}(Exporter_1.Exporter));
exports.JS = JS;
